package li.earth.urchin.twic.xml;

import org.junit.Test;
import org.junit.Before;
import static org.junit.Assert.*;

import java.util.Arrays;

import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import org.xml.sax.SAXException;

public class XMLEventGeneratorTests {
	
	private XMLEventGenerator gen;
	private CollectingListener collector;
	
	@Before
	public void setUp() throws SAXException
	{
		collector = new CollectingListener();
		gen = new XMLEventGenerator(collector);
	}
	
	@Test
	public void testParseMinimalDocument() throws Exception {
		Reader in = new StringReader("<foo/>");
		gen.parse(in);
		assertCollectorContains("startdoc", "item [foo] {} ", "enddoc");
	}
	
	@Test
	public void testParseDocumentWithXMLDeclaration() throws Exception {
		Reader in = new StringReader("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<foo/>");
		gen.parse(in);
		assertCollectorContains("startdoc", "item [foo] {} ", "enddoc");
	}
	
	@Test
	public void testParseDocumentWithNestedElement() throws Exception {
		Reader in = new StringReader("<foo><bar/></foo>");
		gen.parse(in);
		assertCollectorContains("startdoc", "start [foo] {}", "item [foo, bar] {} ", "end [foo]", "enddoc");
	}
	
	@Test
	public void testParseDocumentWithNestedElementAndWhitespace() throws Exception {
		Reader in = new StringReader("<foo> <bar> </bar> </foo>");
		gen.parse(in);
		assertCollectorContains("startdoc", "start [foo] {}", "item [foo, bar] {} ", "end [foo]", "enddoc");
	}
	
	@Test
	public void testParseDocumentWithText() throws Exception {
		Reader in = new StringReader("<foo>bar</foo>");
		gen.parse(in);
		assertCollectorContains("startdoc", "item [foo] {} bar", "enddoc");
	}
	
	@Test
	public void testParseDocumentWithAttributes() throws Exception {
		Reader in = new StringReader("<foo baz='1' bar='2'/>");
		gen.parse(in);
		assertCollectorContains("startdoc", "item [foo] {baz=1, bar=2} ", "enddoc");
	}
	
	@Test
	public void testParseEclipseClasspathFile() throws Exception {
		InputStream in = getClass().getResourceAsStream("eclipse-classpath.xml");
		gen.parse(in);
		assertCollectorContains("startdoc", "start [classpath] {}",
			"item [classpath, classpathentry] {kind=src, path=src} ",
			"item [classpath, classpathentry] {kind=src, output=bin-test, path=test} ",
			"item [classpath, classpathentry] {kind=con, path=org.eclipse.jdt.launching.JRE_CONTAINER/org.eclipse.jdt.internal.debug.ui.launcher.StandardVMType/J2SE-1.5} ",
			"item [classpath, classpathentry] {combineaccessrules=false, kind=src, path=/TestDependingProject} ",
			"item [classpath, classpathentry] {combineaccessrules=false, exported=true, kind=src, path=/TestProject} ",
			"item [classpath, classpathentry] {exported=true, kind=var, path=TEST_HOME/lib/examplelib.jar} ",
			"item [classpath, classpathentry] {kind=lib, path=lib/examplelib.jar} ",
			"item [classpath, classpathentry] {kind=output, path=bin} ",
			"end [classpath]", "enddoc");
	}
	
	@Test
	public void testParseEclipseProjectFile() throws Exception {
		InputStream in = getClass().getResourceAsStream("eclipse-project.xml");
		gen.parse(in);
		assertCollectorContains("startdoc", "start [projectDescription] {}",
			"item [projectDescription, name] {} TestProject",
			"item [projectDescription, comment] {} ",
			"item [projectDescription, projects] {} ",
			"start [projectDescription, buildSpec] {}",
			"start [projectDescription, buildSpec, buildCommand] {}",
			"item [projectDescription, buildSpec, buildCommand, name] {} org.eclipse.jdt.core.javabuilder",
			"item [projectDescription, buildSpec, buildCommand, arguments] {} ",
			"end [projectDescription, buildSpec, buildCommand]",
			"end [projectDescription, buildSpec]",
			"start [projectDescription, natures] {}",
			"item [projectDescription, natures, nature] {} org.eclipse.jdt.core.javanature",
			"end [projectDescription, natures]",
			"end [projectDescription]", "enddoc");
	}
	
	@Test
	public void testParseEclipseLaunchFile() throws Exception {
		InputStream in = getClass().getResourceAsStream("eclipse-launch.xml");
		gen.parse(in);
		assertCollectorContains("startdoc", "start [launchConfiguration] {type=org.eclipse.jdt.launching.localJavaApplication}",
			"start [launchConfiguration, listAttribute] {key=org.eclipse.debug.core.MAPPED_RESOURCE_PATHS}",
			"item [launchConfiguration, listAttribute, listEntry] {value=/TestProject/src/org/example/Foo.java} ",
			"end [launchConfiguration, listAttribute]",
			"start [launchConfiguration, listAttribute] {key=org.eclipse.debug.core.MAPPED_RESOURCE_TYPES}",
			"item [launchConfiguration, listAttribute, listEntry] {value=1} ",
			"end [launchConfiguration, listAttribute]",
			"item [launchConfiguration, stringAttribute] {key=org.eclipse.jdt.launching.MAIN_TYPE, value=org.example.Foo} ",			
			"item [launchConfiguration, stringAttribute] {key=org.eclipse.jdt.launching.PROGRAM_ARGUMENTS, value=foo bar baz} ",			
			"item [launchConfiguration, stringAttribute] {key=org.eclipse.jdt.launching.PROJECT_ATTR, value=TestProject} ",			
			"end [launchConfiguration]", "enddoc");
	}
	
	private void assertCollectorContains(String... events) {
		assertEquals(Arrays.asList(events), collector.events());
	}
	
}
